; Msgbox_mfk creates a custom message box.
; It's best feature is that it permits custom button names - unlike MsgBox, which uses
; predetermined button names. It is a variation on code from
; http://www.autohotkey.com/forum/topic15518.html&highlight=change+msgbox+button and
; http://www.autohotkey.com/forum/topic10248.html
; I changed the code formatting, removed the fontName option, and removed
; font size, BG color setting, button height.
; I also added a "bw" button width option and tweaked the box's margins.
; Options to consider are:
; -Owner
; -Toolwindow
; +AlwaysOnTop
; -Caption
; +0x800000
; [space]bold[space],  [space]itialic[space]

Msgbox_mfk(message="",Options="",Title="",GuiNumber=66)
{ 
    global MessageBoxIcon,MessageBoxText,MessageBoxButton1,MessageBoxButton2 
    ,MessageBoxButton3,MessageBoxButtonName1,MessageBoxButtonName2,MessageBoxCheckbox 

    Gui, %GuiNumber%:Default 
    Gui, Destroy 
    autotrim = %a_autotrim%   ;store it so we can reset it later 
    AutoTrim Off   ;necessary because of the "Options" param 
    Options = %a_space%%Options%%a_space%	;leading and trailing spaces (for StrEnd()) 
    If message = 
       message = Press okay to continue.	;this is the default for the normal Msgbox command also 

    ;the "Buttons" option can be numeric (1-3) or a string seperated by pipes (|). 
    ;the default is "Okay" 
    If (InStr(Options, "Buttons") = 0 OR InStr(Options, "Buttons1")) 
       buttons = Okay 
    else IfInString, Options, Buttons2 
       buttons = Okay|Cancel 
    else IfInString, Options, Buttons3 
       buttons = Yes|No|Cancel 
    else 
     buttons := StrEnd(Options,"buttons") 

    ;If Owner is unspecified, use A_Gui (if non-blank) 
    ;else, use Gui#1 as owner (unless -Owner" is present in Options) 
    ;in any case, we check if the owner exists before using it 
    If ! InStr(Options, "-Owner") { 
       owner := StrEnd(Options,"owner") 
       If ! owner  { 
          If a_gui <> 
             owner = %a_gui% 
          else 
             owner = 1 
           } 
       Gui, %owner%: +LastfoundExist 
       If WinExist() { 
             ;unless specified otherwise, disable the owner. 
             ;note that this causes that neat flickering when 
             ;the user tries to interact with the owner 
             IfNotInString, Options, -Modal 
                Gui, %owner%: +Disabled 
             Gui, +Owner%owner% 
             WinGetPos, x, y, w, h   ;get the position of the owner 
           } 
       else 
          owner = 
    } 

    IfNotInString, Options, -Toolwindow 
       Gui, +Toolwindow 
    IfInString, Options, +AlwaysOnTop 
       Gui, +AlwaysOnTop 
    IfInString, Options, -Caption 
       Gui, -Caption 
    IfInString, Options, +0x800000 
       Gui, +0x800000 
    Gui, +lastfound -SysMenu 
    GuiID := WinExist()   ;used later in the Hotkey command 

    ;get BackgroundColor and TextColor, as well as bold/italic 
    ;BgColor := StrEnd(Options, "Background") 
    ;color := StrEnd(Options, "c") 
    IfInString, Options, %a_space%bold%a_space% 
       bold = bold 
    IfInString, Options, %a_space%italic%a_space% 
       italic = italic 
    ;size := StrEnd(Options, "s") 

    ;change these default values to whatever you  prefer 
    ;If BgColor = 
    ;   BgColor = FFFACD 
    ;If color = 
    ;   color = black 

    ;Gui, Color, %BgColor% 
    ;Gui, Font, s%size% %bold% %italic% c%color% , %fontName% 
    Gui, Font, %bold% %italic% ; c%color%
    ;Gui, Margin, 5, 5
    Gui, Margin, 12, 12 

    icon := StrEnd(Options, "Icon") 
    ;the corresponding icon #'s 
    If Icon = ! 
       i = 2 
    else if icon = ? 
       i = 3 
    else if icon = x 
       i = 4 
    else if icon = i 
       i = 5 

    ;it gets a bit ugly now because of the sectioning with/without the icon... 
    If icon { 
       Gui, Add, Picture, vMessageBoxIcon Section Icon%i%, User32.dll 
       ys = ys 
       xs = xs 
    } 

    width := StrEnd(Options, "w") 
    btnWidth := StrEnd(Options, "bw")
    Gui, Add, Text, vMessageBoxText %ys% w%width%, %message% 

    ;Gui, Font, norm   ;I think buttons with bold font are ugly ;) 

    ;button_h := StrEnd(Options, "h")   ;just in case anyone wants to specify the button height... 
    ;If button_h = 
    ;   button_h = 20 
    ;If button_h not integer 
    ;button_h = 20 

    ;add the buttons: 
    Loop, Parse, buttons, | 
    { 
       StringReplace, b, a_loopfield, _, %a_space%,1 
       numButtons++   ;for the subroutine that positions the buttons 
       If a_index = 1 
        ;  Gui, Add, Button, vMessageBoxButton%a_index% Section h%button_h% %xs% gMessageBox_MessageBoxButtonPushed, %b% 
          Gui, Add, Button, vMessageBoxButton%a_index% Section %xs% w%btnWidth% gMessageBox_MessageBoxButtonPushed, %b% 

          else 
        ;  Gui, Add, Button, vMessageBoxButton%a_index% ys h%button_h% gMessageBox_MessageBoxButtonPushed, %b% 
          Gui, Add, Button, vMessageBoxButton%a_index% ys w%btnWidth% gMessageBox_MessageBoxButtonPushed, %b% 
    } 

    checkboxText := StrEnd(Options,"Checkbox") 
    StringReplace, checkboxText, checkboxText, _, %a_space%,1 
    If checkboxText <> 
       Gui, Add, Checkbox, vMessageBoxCheckbox %xs%, %checkboxText% 

    default_button := StrEnd(Options, "Default") 

    ;if default_button is unspecified, use the last button 
    If default_button = 
       default_button = %numButtons% 

    ;to have NO default button, specify Default0 
    If default_button <> 0 
       GuiControl, +Default, MessageBoxButton%default_button% 
    GuiControl,  Focus, MessageBoxButton%default_button% 

    ;this works for the real Msgbox, so we need it too :) 
    Hotkey, IfWinActive, ahk_id%GuiID% 
    Hotkey, ^c, MessageBox_CopyMessageBoxTextToClipboard 

    Gui, Show, Autosize Hide, %title% 
    Gosub, MessageBox_MoveMessageBoxButtons   ;move the buttons 

    ;if we have no owner, just show the Gui in the middle of the screen 
    ;else, position is in the middle of its owner 
    If owner { 
             Gui, +Lastfound 
             WinGetPos,,,msg_w,msg_h 
             msg_x := x + w/2 - msg_w/2 
             msg_y := y + h/2 - msg_h/2 
             Gui, Show, x%msg_x% y%msg_y%, %title% 
        }   else 
    Gui, Show,, %title% 

    ;this thread can't be critical (otherwise the MessageBox_ButtonPushed thread 
    ;would be buffered endlessly...) 
    Critical Off 
    WinWaitClose      ;wait until the user clicks one of the buttons 

    ;turn off the hotkey and reset autotrim 
    Hotkey, ^c, MessageBox_CopyMessageBoxTextToClipboard, Off 
    Autotrim %autotrim% 
    If MessageBoxCheckbox = 1 
       ErrorLevel = 1 
    else 
       ErrorLevel = 0 
    If owner 
       Gui %owner%:Default 
    else 
       Gui, 1:Default 
    return return_value   ;return the return value set by the ButtonPushed thread 

    MessageBox_MessageBoxButtonPushed: 
    ;if we've been asked to redraw: 
    StringRight, num, a_guicontrol, 1   ;the button # 
    GuiControlGet, return_value,,%a_guicontrol% 
    StringReplace, return_value, return_value, & ;get rid of the ampersand 
    Gui, Submit 
    If owner 
       Gui, %owner%: -Disabled 
    Gui, Destroy 
    return 

    ;the hotkey thread 
    MessageBox_CopyMessageBoxTextToClipboard: 
     clipboard = %message% 
    return 

    ;center the buttons: 
    MessageBox_MoveMessageBoxButtons: 
    Gui %GuiNumber%:Default 
    WinGetPos,,,gw 
    gw -= 6 
    GuiControlGet, txt, Pos, MessageBoxText 
    If ! icon 
       GuiControl, Move, MessageBoxText, % "x" gw/2 - txtW/2 

    Loop 3 
     GuiControlGet, p%a_index%, Pos, MessageBoxButton%a_index% 

    If numButtons = 1 
     GuiControl, Move, MessageBoxButton1, % "x" gw/2 - p1w/2 
    else if (numButtons = 2)  { 
     GuiControl, Move, MessageBoxButton1, % "x" gw/2 - ((p1w + p2w + 5)/2) 
     GuiControl, Move, MessageBoxButton2, % "x" gw/2 - ((p1w + p2w + 5)/2) + p1w + 5 
    } else if (numButtons = 3) { 
     GuiControl, Move, MessageBoxButton1, % "x" gw/2 - ((p1w + p2w + p3w + 10)/2) 
     GuiControl, Move, MessageBoxButton2, % "x" gw/2 - ((p1w + p2w + p3w + 10)/2) + p1w + 5 
     GuiControl, Move, MessageBoxButton3, % "x" gw/2 - ((p1w + p2w + p3w + 10)/2) + p1w + p2w + 10 
    }
    return 
} 

;########## this small helper funtion also has to be present ############ 

;this function extracts the trailing characters of a string 
;(for instance "White" from BackgroundWhite etc.) 
StrEnd(O,str)
{ 
    IfNotInString, O, %a_space%%str% 
       return 
    StringMid, e, O, InStr(O, " " str) + StrLen(str) + 1 
    , InStr(O, a_space, "", InStr(O, " " str) + 1) - InStr(O, " " str) - StrLen(str) - 1 
    return e 
}