; "jIDEE-0004.ahk"
; 5:11 PM Friday, 05 September, 2008 
;
; This is an attempt to build a simple IDE for JAVA (based on the cIDEE
; code base).
;
; Copyright (c) 2008 Mithat Konar
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;---------------------------;
;----- preambley stuff -----;
;---------------------------;

#SingleInstance, off
#NoEnv
SendMode Input				; Recommended for new scripts due to its superior speed and reliability.
kAppName = jIDEE						; the name of this script
kVersionString = 0.0.0.4					; script version number
FormatTime, now,,yyyy-MM-dd hh:mm:ss tt	; "now" is the time the script started

;--------------------;
;----- includes -----;
;--------------------;

#include %A_ScriptDir%\inc_GuiStrings.ahk
#include %A_ScriptDir%\inc_HotkeyDef.ahk
#include %A_ScriptDir%\inc_GuiParams.ahk
#include %A_ScriptDir%\inc_FileExtensions.ahk
#include %A_ScriptDir%\inc_Msgbox_mfk.ahk
#include %A_ScriptDir%\inc_Editor.ahk
#include %A_ScriptDir%\inc_AddGraphicButton_2-1.ahk
#include %A_ScriptDir%\inc_WMMOUSEMOVE.ahk
;other includes are found where needed in the remainder of the code

;-----------------------------;
;----- system tray setup -----;
;-----------------------------;
#NoTrayIcon

;---------------------------------;
;----- some global constants -----;
;---------------------------------;

kBinDir = %A_ScriptDir%\rsc\bin						; path to local resource binary stuff
kShowFilePath = %kBinDir%\ShowFile					; path and name of tool used to view output files
kTempBat = tempidee.bat								; name of temporary bat file used for compiling, etc.
kOutputFile = output.log							; name of temporary file used to store compiler, linker, etc. output
kHelpFilePath = "%A_ScriptDir%\jIDEE help.chm"		; path to help file
kCompilerVersionPath = %kBinDir%\compiler_version.bat
; both of the below are assumed to be in gToolPath (which is defined at runtime)
kCompileTool = javac								; name of tool used to compile source code
kInterpreterTool = java								; name of tool used to run bytecode

; ini file global constants
kIniPath = %A_AppData%\jIDEE				; path to settings file
kIniFile = %kIniPath%\settings.ini
kIniRootDirSection = rootdir				; name of section for below
kRootDirKey = dir							; name of key that holds name of default root (project) directory
kToolSection = javatools                          ; name of section for setting tool path stuff
kToolPathKey = path                               ; name of key that holds path to tools
; TODO: add these to the ini file or get rid of them completely?
;kToolCompilerKey = compiler                       ; name of key that holds name of compiler command
;kToolJVMKey = jvm                                 ; name of key that holds name of execute class command files

;----------------------------------;
;----- misc. global variables -----;
;----------------------------------;

gToolPath = 					; path to non-local tools (e.g., compiler/interpreter/etc.)
gCurrentFileName =				; holds last opened filename (with extension)
gTempFileName =					; temp var to facilitate "passing" of filenames amongst subroutines.
gTempDirName =					; temp var to facilitate "passing" of directory names amongst subroutines.
gArgList =						; command-line argument list used when executing/running files
gCompileWindowNum = 0			; holds count of compile windows opened modulo kNumWindows
gGUIWinTitle = %kAppName%   	; title of GUI window

;---------------------------------------;
; ---- initial root directory stuff ----;
;---------------------------------------;
gRootDir =				; default root directory
gosub InitLangTools		; make sure we've got valid Java tools and project dir
if !gRootDir			; if we didn't get a gRootDir after initializing tools...
	IniRead, gRootDir, %kIniFile%, %kIniRootDirSection%, %kRootDirKey%					; if a default ini file does not exist, it will default to nothing
gRootDir := initRootDir(gRootDir)

;-------------------------;
;----- launch action -----;
;-------------------------;
;Create an ImageList and put some standard system icons into it (directory icons in particular):
ImageListID := IL_Create(5)
Loop 5  ; Below omits the DLL's path so that it works on Windows 9x too:
	IL_Add(ImageListID, "shell32.dll", A_Index)

;staticWinState = 1		; used by RefreshIfActivating. initial window state is active
gosub OpenGUI			; launch the GUI

;-----------------------------------------;
;----- persistent Hotkey definitions -----;
;-----------------------------------------;
; These should be used only during debugging
;^#!r::Reload		; ctrl-alt-Windows-R restarts the script.
;^!r::Reload		; ctrl-alt-R restarts the script.
;!r::Reload		; alt-R restarts the script.


;*******************************************;
;*************** Subroutines ***************;
;*******************************************;

;-----------------------------;
;----- GUI windows stuff -----;
;-----------------------------;
OpenGUI:
; Opens a GUI window if it doesn't already exist. If the GUI window already
; exists, activates it.
	if GUIWinExist(gGUIWinTitle, kGUIWinID)
	{
		WinActivate, %gGUIWinTitle% ahk_class AutoHotkeyGUI, %kGUIWinID%
	}
	else
	{
		xpos = %kDefaultOrigX%
		ypos = %kDefaultOrigY%
		gosub BuildGUIMenuBar
		Gui, -Resize			; enable/diable resizing

		; Root directory navigation
		AddGraphicButton("BtnChangeRootDir", A_ScriptDir . "\rsc\images\sitemap_color.bmp", "section h36 w36 gBtnChangeRootDir", 16,16)		
		Gui, font, bold
		Gui, font, s12 c555555
		Gui, Add, Text, section ys w%kDirPathWidth% vTxtRootDir gNullSR
		Gui, font
		Gui, font, c868686
		Gui, Add, Text, y+1 w%kDirPathWidth% vTxtRootPath, %gRootDir%
		Gui, font
		AddGraphicButton("BtnExploreProjectDir", A_ScriptDir . "\rsc\images\folder_explore.bmp", "ys h36 w36 gExploreProjectDir", 16,16)
		AddGraphicButton("BtnCompileRoot", A_ScriptDir . "\rsc\images\lightning.bmp", "ys x+4 h36 w36 gBtnCompileRoot", 16,16)
		
		; Directory tree groupbox
		Gui, Add, GroupBox, section xm w%kDirGrpBoxWidth% h%kFileGrpBoxHeight%, %kDirGrpBoxTitle%
		; Directory browser column
		Gui, Add, TreeView, xs+11 ys+20 vDirTree r20 w%kDirTreeWidth% h%kFileListHeight% gDirTree_event ImageList%ImageListID%		

		; Files groupbox
		Gui, Add, GroupBox, section ys w%kFileGrpBoxWidth% h%kFileGrpBoxHeight%, %kFilesGrpBoxTitle%
		; Editable/compilable files column
		Gui, Add, ListView, -multi sort +NoSortHdr section section xs+11 ys+20 w%kFileListWidth% h%kFileListHeight% AltSubmit vEditableListView gEditableList_event, %kEditableHeader%
		; Compiled files column
		Gui, Add, ListView, -multi sort +NoSortHdr x+2 ys w%kFileListWidth% h%kFileListHeight% AltSubmit vCompiledListView gCompiledList_event, %kCompiledHeader%
		
		; ; Buttons! (disabled because they really clutter the UI)
		; AddGraphicButton("BtnNewFile", A_ScriptDir . "\rsc\images\page_white_add.bmp", "ys h24 w24 AltSubmit vBtnNewFile gBtnNewFile", 16,16)
		; AddGraphicButton("BtnExploreSourceDir", A_ScriptDir . "\rsc\images\folder_explore.bmp", "x+4 h24 w24 gExploreWorkingDir", 16,16)
		
		; Magic hidden ID text used to unambiguously identify this window.
		Gui, Add, Text, Hidden w0 h0, %kGUIWinID%
		Gui, Add, StatusBar, vstatbar_path gExploreWorkingDir,
		gTempDirName = %gRootDir%
		gosub SetRootDir	; set directories and show the GUI

		; Add hotkeys
		Hotkey, IfWinActive, %gGUIWinTitle% ahk_class AutoHotkeyGUI, %kGUIWinID%
		Hotkey, %HKChangeRootDir%, ChangeRootDir
		Hotkey, %HKExploreRootDir%, ExploreProjectDir
		Hotkey, %HKExploreWorkingDir%, ExploreWorkingDir
		Hotkey, %HKNew%, CreateNewFile
		Hotkey, %HKRefresh%, RefreshAll
		Hotkey, %HKBuildProject%, CompileRootDir
		;Hotkey, %HKClose%, ButtonCancel
		Hotkey, %HKQuit%, Quit
		Hotkey, %HKHelp%, Help

		; Launch update timer
		SetTimer, RefreshIfActivating, 500
	}
	return

;--------------------------------------;
;----- GUI windows special labels -----;
;--------------------------------------;

;GuiEscape:					; Cancel and Escape destroy the GUI window.
ButtonCancel:
	gosub DestroyGUIWindow
	return

GuiClose:					; Close destroys the GUI window
	gosub DestroyGUIWindow
	gosub Quit				; and Quits the App. if line is uncommented
	return

GuiSize:
; If restoring or resizing, refresh lists
; TODO: needed under new "automatic refresh" paradigm?
	if ErrorLevel = 0	; if restoring or resizing
	{
		gosub RefreshLists
	}
	return

;------------------------------;
;----- dialog boxes, etc. -----;
;------------------------------;
Help:
; Displays help.
	Run, %kHelpFilePath%
	return

About:
; Displays or activates an About box.
	IfWinExist, %kAboutMsgBoxTitle% ahk_class #32770, %kAboutStr%
		WinActivate, %kAboutMsgBoxTitle% ahk_class #32770, %kAboutStr%
	Else
		Gui +OwnDialogs
		MsgBox, 64, %kAboutMsgBoxTitle%, %kAboutStr%  %gHwndAbout%	; IconAsterisk
	return

CompilerVersion:
; Displays version information for compiler
	Run, "%kCompilerVersionPath%" "%gToolPath%\%kInterpreterTool%"
	return

WWWpage:
; launches jIDEE homepage in default browser
	Run, %kHomePage%
	return

;---------------------------;
;----- helper routines -----;
;---------------------------;
#Include %A_ScriptDir%\inc_ListViews.ahk	; contians list making and handling stuff
#Include %A_ScriptDir%\inc_DirTreeView.ahk	; contians directory tree making and handling stuff

Quit:
; Er.... does the obvious.
	IfNotExist, %kIniPath%
		FileCreateDir, %kIniPath%
	IniWrite, %gRootDir%, %kIniFile%, %kIniRootDirSection%, %kRootDirKey%
	IniWrite, %gToolPath%, %kIniFile%, %kToolSection%, %kToolPathKey%  ; get stuff from ini file
	if GUIWinExist(gGUIWinTitle, kGUIWinID)
		gosub DestroyGUIWindow
	ExitApp
	return

BuildGUIMenuBar:
; Builds the GUI window's menu bar
	Menu, FileMenu, Add, %kMenuDirChange%, ChangeRootDir
	Menu, FileMenu, Add, %kMenuFileNew%, CreateNewFile
	Menu, FileMenu, Add
	Menu, FileMenu, Add, %kMenuDirExploreRoot%, ExploreProjectDir
	Menu, FileMenu, Add, %kMenuDirExploreWork%, ExploreWorkingDir
	Menu, FileMenu, Add
	Menu, FileMenu, Add, %kMenuFileRefresh%, RefreshAll
	Menu, FileMenu, Add
	Menu, FileMenu, Add, %kMenuFileExit%, Quit
	Menu, BuildMenu, Add, %kMenuBuildCompileRoot%, CompileRootDir
	Menu, HelpMenu, Add, %kMenuHelpHelp%, Help
	Menu, HelpMenu, Add, %kMenuHelpAbout%, About
	Menu, HelpMenu, Add, %kMenuHelpHomepage%, WWWpage
	Menu, HelpMenu, Add
	Menu, HelpMenu, Add, %kMenuHelpCompilerVersion%, CompilerVersion
	Menu, HelpMenu, Add, %kConfigJavaTools%, SetJavaToolDir

	; Create the menu bar by attaching the sub-menus to it:
	Menu, MyMenuBar, Add, %kFileMenuTxt%, :FileMenu
	Menu, MyMenuBar, Add, %kBuildMenuTxt%, :BuildMenu
	Menu, MyMenuBar, Add, %kHelpMenuTxt%, :HelpMenu

	; Attach the menu bar to the window:
	Gui, Menu, MyMenuBar
	return

DestroyGUIWindow:
	Gui, Destroy
	return

DeletegTempFileName:
; Deletes gTempFileName first asking for confirmation
	strDelName = %gTempFileName%	; buffer gTempFileName in case it gets lost before MsgBox is dismissed
	Gui +OwnDialogs
	MsgBox, 292, %kFileListConfirmDeleteTitle%, %kFileListConfirmDelete% "%strDelName%"?	; Yes/No+IconQuestion+2ndBtnDefault
	IfMsgBox Yes
	{
		FileDelete, %strDelName%	; delete the file
		gosub RefreshLists			; rebuild lists to remove deleted file
	}
	return

CreateNewDir: ;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; TODO: this subroutine is mostly obsolete.
;       need a way to create new working directories
;       creating new root directories can be handled through the ChangeRootDir subroutine
	return

ChangeRootDir:
; prompt for and set new root dir, update "root dir" text on main gui,
; update (rebuild) directory lists and update (rebuild) file lists
; TODO move GUI sting to GUI string files
	Gui +OwnDialogs	; Forces user to dismiss the following dialog before using main window.
	FileSelectFolder, gTempDirName, *%gRootDir%, 1, %kChangeRootFolder%
	if not gTempDirName		; The user canceled the dialog.
		return
	; if user selected a root drive, give message and return
	SplitPath, gTempDirName, projectRootName,,,, 	; if user selected a root drive, projectRootName will be blank 
	if projectRootName
		gosub SetRootDir
	else
	{
		MsgBox, 48, %kRootDrivesNotAllowedTitle%, %kRootDrivesNotAllowed%	; error
		gosub ChangeRootDir			; recurse.
	}
	return

SetRootDir:
; set the root (and working) directory to gTempDirName,
; update "root dir" text on main gui and compile folder button,
; update (rebuild) directory lists and update (rebuild) file lists
	; TODO: buffer gTempDirName just in case some other thread hijacks it?
	;       delete temp files in the current directory?
	gRootDir = %gTempDirName%
	tmpDirtext := TrimProjectName(gRootDir, kDirNumChars)		; tmpDirtext has truncated name
	GuiControl,,TxtRootDir, %tmpDirtext%						; update project name
	tmpDirPath := TrimProjectPath(gRootDir, kDirNumChars)		; tmpDirPath has pretty path
	GuiControl,,TxtRootPath, [ %tmpDirPath% ]					; update filepath text
	;TxtRootDir_TT = %tmpDirPath% 
	;gGUIWinTitle = %kAppName% [%tmpDirtext%]
	GuiControl,,BtnCompileRoot, %btnLbl_BtnCompileRoot% "%tmpDirtext%"	; not needed with graphic buttons
	Gui, Show, , %gGUIWinTitle%		; (re)show the GUI window
	OnMessage(0x200, "WM_MOUSEMOVE")
	gosub BuildDirTree
	gosub SetWorkDir
	return
	
SetWorkDir:
; set the working directory to gTempDirName, and update (rebuild) file lists
	; delete temp files in the current directory?
	SetWorkingDir, %gTempDirName%
	SB_SetText(gTempDirName)		; update status bar text
	gosub RefreshLists
	return	

ExploreWorkingDir:
; launch working directory in Windows Explorer
	Run, "%A_WorkingDir%"
	;gosub RefreshLists
	return

ExploreProjectDir:
; launch project root directory in Windows Explorer
	Run, "%gRootDir%"
	return

CreateNewFile:
; create a new editable file and open it in the external editor
	; prompt for new file name
	Gui, +OwnDialogs
	InputBox, strNewFileName, %kNewFileTitle%, %kNewFileTxt%,,kNominalDialogW,kNominalDialogH,,,,, %kNewFileDefaultTxt%
	; check for errors:
	if ErrorLevel <> 0		; if we canceled
		return				; get out of here
	if !strNewFileName		; if we didn't get a filename
		return				; get out of here
	SplitPath, strNewFileName,,, fileExt	; Get the file's extension.
	if fileExt not in %kEditableExts%
	{
		MsgBox, 16, %kWrongExtensionTitle%, "%fileExt%" %kWrongExtensionTxt% %kEditableExts%	; IconError
		gosub CreateNewFile		; recursive fix condition
		return
	}
	IfExist, %strNewFileName%	; if strNewFileName already exists
	{
		MsgBox, 308, %kFileExistsTitle%, %kFileExists1% "%strNewFileName%" %kFileExists2%	; yes/no, IconExclamation, 2ndDefault
		IfMsgBox, No			; if no...
		{
			gosub CreateNewFile	; recursive fix condition
			return
		}
		IfMsgBox, Yes			; if yes (tested explicity in case MsgBox options are expanded in the future)...
			FileDelete, %strNewFileName%	; delete file so that a fresh file will be made by FileAppend
	}
	; if you made it here, there are no errors -- make a new file with the given name
	FormatTime, creationTime,,yyyy-MM-dd hh:mm:ss tt
	FileAppend, /*`n * %strNewFileName%`n * created: %creationTime%`n * author: %A_UserName%`n */`n`n`n, %strNewFileName%	; add comments with file's name (TODO: make these konstants). Extra CR at end of file makes some compilers happier.
	gTempFileName = %strNewFileName%
	gosub EditableList_eventOpen	; and open it in external editor
	gosub BuildEditableList			; rebuild list to add it to the file list
	return

DefaultAction:
; TODO use this for something useful
	return

RefreshIfActivating:
; this should be used within a polling scheme (e.g. with a timer) to affect periodic refreshes
; of the file lists only when needed.
; if previous state was *not-active* and present state is *active*, refresh the file lists (and set
; state to active
; otherwise set the state as approp.
	IfWinActive, %gGUIWinTitle% ahk_class AutoHotkeyGUI, %kGUIWinID%
	{
		IfEqual, staticWinState, 0		; if we are coming out of an inactive state
		{
			;SoundBeep, 3000, 250		; DEBUG only
			;msgbox DEBUG: RefreshIfActivating refreshing
			gosub RefreshLists
			staticWinState = 1
			; TODO: compare present file lists with actual file list (from drive) and update only if different
		}
	}
	Else
	{
		staticWinState = 0
	}
	return

RefreshAll:
; refreshes directory tree and file lists
	gosub BuildDirTree
RefreshLists:
; refreshes file lists
	gosub BuildEditableList
	gosub BuildCompiledList
	;SoundBeep, 500, 300
	return

CompileTempFile:
; compiles gTempFileName and refreshes the file lists
; gTempFileName is assiumed to be in A_WorkingDir
    SplitPath, gTempFileName,,, fileExt		; Get the file's extension.
    if fileExt not in %kCompilableExts%		; if it's not a compilable source file
        return                       		; shouldn't get here, but you never know
    SplitPath, gTempFileName,,,fileExt,fname	; Get the file's extension and name.
    IfExist, %fname%.%kCompiledExts%		; delete old object file (TODO: determine if this is the best behavior)
        FileDelete, %fname%.%kCompiledExts%
    FileDelete, %A_WorkingDir%\%kTempBat%					; first delete the old batch file in case it didn't get deleted for some reason
    FileDelete, %A_WorkingDir%\%kOutputFile%	; ditto the output file
	gTempDirName := LeafName(gRootDir)		; extract leaf node of root path
	gTempCompileCommand = "%gToolPath%\%kCompileTool%" -classpath "%gRootDir%" %gTempFileName%
	gosub MakeBATandCompile
	return

CompileRootDir:
	; temporarily set working dir to root and compile *.java.
	gTempOldWorkingDir = %A_WorkingDir%		; save original working dir
	gTempDirName = %gRootDir%				; set working dir to root
	gosub SetWorkDir						; ditto
	FileDelete, %kTempBat%					; first delete the old batch file in case it didn't get deleted for some reason
    FileDelete, %A_WorkingDir%\%kOutputFile%	; ditto the output file
	gTempFileName = *.java
	gTempDirName := LeafName(gRootDir)		; extract leaf node of root path
	gTempCompileCommand = "%gToolPath%\%kCompileTool%" %gTempFileName%
	gosub MakeBATandCompile
	gTempDirName = %gTempOldWorkingDir%		; reset original working dir
	gosub SetWorkDir						; ditto
	return

MakeBATandCompile:
; create a batch file in the working directory that will be used to echo an
; informative message to the window and start the compile, redirecting
; stderr to the output file.
; requires the following to be set:
; 	gTempCompileCommand - the command line instruction used in the compile (including filename and path)
;	gTempFileName - used only for UI: "Compiling %gTempFileName%"
;	gTempDirName - used only for UI:  "in folder %gTempDirName%"
; uses the following konstants:
; 	kCompilerTitle
; 	kTempBat
; 	kOutputFile
;	kCompileTool
;	misc. konstants associated with ShowFile
; uses the following as static variables
;	gCompileWindowNum

    FileAppend,               ; now make the new batch file
    (
@echo  off
REM This is a temporary file. Delete this file if it has not been deleted automatically.
title %kCompilerTitle%
echo Compiling %gTempFileName% with %kCompileTool% . . .
%gTempCompileCommand% 2>> "%A_WorkingDir%\%kOutputFile%"
    ), %kTempBat%

    ; start building output file    
    ;theDateTime := DateStrFine()	; start time (or close to it)
    ; write output file header
    FileAppend,               
    (
Compiling: "%gTempFileName%" with %kCompileTool%
Project folder: "%gTempDirName%"
------------------------------------------------------------

    ), %A_WorkingDir%\%kOutputFile%
    
    ; compile and append errors to output file
    RunWait, %kTempBat%      ; run the batch file
	
	theDateTime := DateStr()	; end time (or close to it)

    ; write output file footer
    FileAppend,               
    (

Done! [%theDateTime%]
    ), %A_WorkingDir%\%kOutputFile%
 
    titleStr = %gTempFileName% [%theDateTime%]             ; string to be used in title of output file listing
	gosub BuildCompiledList	; rebuild file list to reflect possible file changes
    
    xpos  := A_ScreenWidth - kShowFileWidth - 40 - 20*gCompileWindowNum
    gCompileWindowNum := mod(gCompileWindowNum+1, kNumWindows)
	
	IfExist, %kOutputFile%
    { 
		run, "%kShowFilePath%" %kOutputFile% "%titleStr%" %kShowFileWidth% %kShowFileHeight% %xpos% 0 s9 "Lucida Console" FFFFFFS
        WinWait, %titleStr%,,5,,
        if ErrorLevel   ; i.e. it's not blank or zero.
	       MsgBox, I seem to be having a problem opening the compiler results.
        else
        {
	       FileDelete, %kTempBat%   ; delete the batch file
           FileDelete, %kOutputFile%   ; delete the output
        }
    }
	return

NullSR:
; does nothing
	return
	
;----------------------------------;
;---- setting tool paths stuff ----;
;----------------------------------;
; moved to...
#Include %A_ScriptDir%\inc_ConfigTools.ahk

;---------------------------;
;----- Button handlers -----;
;---------------------------;
BtnNewDir:
; prompt for name of new directrory to create in root folder
	gosub CreateNewDir
	return

BtnChangeRootDir:
; prompt for and set new working dir, update (rebuild) file lists and update working dir text on main gui
	gosub ChangeRootDir
	return

BtnNewFile:
; create a new source file
	gosub CreateNewFile
	return

BtnCompileRoot:
; recursively compile a whole directory starting from root
    gosub CompileRootDir
    return

;*****************************************;
;*************** Functions ***************;
;*****************************************;

initRootDir(rootDir)
; if rootDir does not exist, initialize it.
; returns the rootDir directory
{
	global gRootDir
	IfExist, %rootDir%
		return %rootDir%
	else
	{	
		gosub InitNewRootName
		return %gRootDir%
	}
}

GUIWinExist(GUIwinTitle, winID)
; Returns 1 if the GUI window exists, 0 otherwise.
; I made this a function in case I discover a better way to positively identify
; a given window.
{
	IfWinExist, %GUIwinTitle% ahk_class AutoHotkeyGUI, %winID%
		return 1
	return 0
}

ShowHelpBox(msgTitle, ahkClass, msg)
; Shows or activates a help-style message box
{
	IfWinExist, %msgTitle% ahk_class %ahkClass%, %msg%
	{
		WinActivate, %msgTitle% ahk_class %ahkClass%, %msg%
	}
	Else
	{
		Gui +OwnDialogs
		MsgBox, 64, %msgTitle%, %msg%		; IconAsterisk
		;MsgBox, 0, %msgTitle%, %msg%		; no icon
	}
	return
}

TrimLeftWithElipsis(inString, len)
; If inString is longer than len, returns inString trimmed to (len - 3) characters
; and adds elipsis in front; otherwise returns inString
{
	inStrLen := StrLen(inString)
	trimLen := inStrLen - (len - 3)
	if trimLen > 0
	{
		StringTrimLeft, inString, inString, %trimLen%
		inString = ...%inString%
	}

	return inString
}

TrimProjectPath(projectPathStr, len)
; projectPathStr is assumed to be a path to a project folder. TrimProjectPath will return a
; string reflecting the projectPathStr that is formated in a friendly way so that the returned
; strings length is less than or equal to len.
{
;	global gRootDir

	;if the path begins with the default working directory, then trim off everything but "IDEE projects" or whatever
;	if InStr(projectPathStr, gRootDir)
;	{
;		containerDir := LeafName(gRootDir)	; Store "IDEE projects" (or whatever) name in containerDir
;		containerDir = %containerDir%
;		StringReplace, projectPathStr, projectPathStr, %gRootDir%, %containerDir%	; replace full path with shorter name
;	}
	;else, if the path begins with the user's My Documents, then trim off everything but "My Documents"
;	else
	if InStr(projectPathStr, A_MyDocuments)
	{
		containerDir := LeafName(A_MyDocuments)	; Store "My Documents" name in containerDir
		StringReplace, projectPathStr, projectPathStr, %A_MyDocuments%, %containerDir%	; replace full path with shorter name
	}
	;else, if the path begins with the user's Desktop, then trim off everything but "Desktop"
	else if InStr(projectPathStr, A_Desktop)
	{
		containerDir := LeafName(A_Desktop)	; Store "Desktop" name in containerDir
		StringReplace, projectPathStr, projectPathStr, %A_Desktop%, %containerDir%	; replace full path with shorter name
	}
	return TrimLeftWithElipsis(projectPathStr, len)
	; TODO: trim at "\" if possible
}

TrimProjectName(projectPathStr, len)
; projectPathStr is assumed to be a path to a project folder. TrimProjectName will return a
; string reflecting the "name of the project" which is assumned to be the last folder name in
; projectPathStr. The returned string will be trimmed to have a length less than or equal to len.
{
	projectName := LeafName(projectPathStr)	; get the name of the last folder
	; if projectName is blank, it means projectPathStr is the root drive
	If projectName
		projectName = %projectName%		; no need to do anything
	else
		projectName = %projectPathStr%

	return TrimLeftWithElipsis(projectName, len)
}

ContainingPath(pathStr)
; Returns a string that is the path containing pathStr. (Assumes pathStr is a path.)
; Example: if pathStr is C:\Foo\FooBar, ContainingPath(pathStr) will return C:\Foo
;		  if pathStr is C:\Foo, ContainingPath(pathStr) will return C:\
;		  if pathStr is C:\, ContainingPath(pathStr) will return C:
{
	pos := InStr(pathStr, "\" , false, 0)		; find 1st occurence of "\" from right end
	newPathStr := SubStr(pathStr, 1, pos-1)		; trim everything from there (inclusive)
	return newPathStr
}

LeafName(pathStr)
; Returns a string that is the final directory in pathStr. (Assumes pathStr is a path.)
; Example: if pathStr is C:\Foo\FooBar, LeafName(pathStr) will return FooBar
;		  if pathStr is C:\Foo, LeafName(pathStr) will return Foo
;		  if pathStr is C:\, LeafName(pathStr) will return an empty string
{
	pos := InStr(pathStr, "\" , false, 0)		; find 1st occurence of "\" from right end
	StringTrimLeft, directoryStr, pathStr, pos	; trim everything to the left (including "\")
	return directoryStr
}

DateStr()
; Returns the current date and time
{
	FormatTime, theDateTime,,yyyy-MM-dd hh:mm:ss tt
	return theDateTime
}

DateStrFine()
; Returns the current date and time with milliseconds
{
	theDateTime = %A_YYYY%-%A_MM%-%A_DD% %A_Hour%:%A_Min%:%A_Sec%.%A_Msec%
	return theDateTime
}
